import unittest
from gsch.paper import Paper


class TestPaper(unittest.TestCase):

    def test___init__(self):
        paper = Paper('Awesome Study in 2017', ['John Smith'], '2017', 10,
                      'https://smith2017.pdf')
        self.assertEqual(paper.title, 'Awesome Study in 2017')
        self.assertEqual(paper.authors, ['John Smith'])
        self.assertEqual(paper.year, '2017')
        self.assertEqual(paper.cited_by, 10)
        self.assertEqual(paper.url, 'https://smith2017.pdf')


if __name__ == "__main__":
    unittest.main()



class Paper:

    def __init__(self, title, authors=[], year=None, cited_by=None, url=None,
                 snippets=[]):
        self.title = title
        self.authors = authors
        self.year = year
        self.cited_by = cited_by
        self.url = url
        self.snippets = snippets



import unittest
from pyquery import PyQuery
from gsch.agent import Agent
from gsch.option import Option


class TestAgent(unittest.TestCase):

    def test__set_url_for(self):
        agent = Agent()
        keywords = ['aaa', 'bbb']

        option = Option()
        url = agent._set_url_for(keywords, option)
        expected = 'https://scholar.google.com/scholar?q=aaa+bbb'
        self.assertEqual(url, expected)

        option = Option(start=10)
        url = agent._set_url_for(keywords, option)
        expected = 'https://scholar.google.com/scholar?q=aaa+bbb&start=10'
        self.assertEqual(url, expected)

    def test__extract_papers_from(self):
        html = """
        <html>
          <div class="gs_r gs_or gs_scl">
            <div class="gs_ggs gs_fl">
              <a href="https://smith2018.pdf"></a>
            </div>
            <div class="gs_ri">
              <h3><a>Awesome Study in 2018</a></h3>
              <div class="gs_a">John Smith - conference, 2018 - site_url</div>
              <div class="gs_rs">This paper introduces my awesome study.</div>
              <div class="gs_fl">Cited by 100</div>
            </div>
          </div>
        </html>
        """

        agent = Agent()
        pq_html = PyQuery(html)
        papers = agent._extract_papers_from(pq_html)
        self.assertEqual(papers[0].url, 'https://smith2018.pdf')
        self.assertEqual(papers[0].title, 'Awesome Study in 2018')
        self.assertEqual(papers[0].authors, ['John Smith'])
        self.assertEqual(papers[0].year, '2018')
        self.assertEqual(papers[0].cited_by, '100')
        self.assertEqual(papers[0].snippets,
                         'This paper introduces my awesome study.')


if __name__ == "__main__":
    unittest.main()



import re
from gsch.paper import Paper
from gsch.option import Option
from pyquery import PyQuery


class Agent():

    SCHOLAR_URL = 'https://scholar.google.com/scholar'

    REGEXP_FOR_AUTHORS = re.compile(r'<a.+?>|</a>|<b>|</b>')
    REGEXP_FOR_YEAR = re.compile(r'([0-9]{4}$)')
    REGEXP_FOR_CITED_BY = re.compile(r'Cited by ([0-9]+)')

    def __init__(self):
        pass

    def search(self, keywords, option=Option()):
        url = self._set_url_for(keywords, option)
        pq_html = PyQuery(url)
        papers = self._extract_papers_from(pq_html)
        return papers

    def _set_url_for(self, keywords, option):
        url = self.SCHOLAR_URL + '?q=' + '+'.join(keywords)
        if option.start != 0:
            url += '&start=' + str(option.start)
        if option.year_low is not None:
            url += '&as_ylo=' + option.year_low
        if option.year_high is not None:
            url += '&as_yhi=' + option.year_high
        return url

    def _extract_papers_from(self, pq_html):
        papers = []
        for div in pq_html.find('div.gs_r.gs_or.gs_scl'):
            pq_div = PyQuery(div)
            paper = self._extract_paper_from(pq_div)
            papers.append(paper)
        return papers

    def _extract_paper_from(self, pq_div):
        title = self._extract_title_from(pq_div)
        authors, year = self._extract_authors_and_year_from(pq_div)
        cited_by = self._extract_cited_by_from(pq_div)
        url = self._extract_url_from(pq_div)
        snippets = self._extract_snippets_from(pq_div)
        return Paper(title, authors, year, cited_by, url, snippets)

    def _extract_title_from(self, pq_div):
        title = pq_div.find('div.gs_ri h3 a').text()
        return title

    def _extract_authors_and_year_from(self, pq_div):
        array = re.split('[  ]- ', pq_div.find('.gs_a').html())
        authors = [re.sub(self.REGEXP_FOR_AUTHORS, '', author)
                   for author in array[0].split(', ')]
        match = re.search(self.REGEXP_FOR_YEAR, array[1])
        year = match.group(1) if match else None
        return authors, year

    def _extract_cited_by_from(self, pq_div):
        html = pq_div.find('div.gs_ri div.gs_fl').html()
        match = re.search(self.REGEXP_FOR_CITED_BY, html)
        if not match:
            return ''
        else:
            return match.group(1)

    def _extract_url_from(self, pq_div):
        url = pq_div.find('div.gs_ggs.gs_fl a').attr('href')
        return url

    def _extract_snippets_from(self, pq_div):
        snippets = pq_div.find('div.gs_ri div.gs_rs').html()
        snippets = re.sub(r'<[\w/]+>', '', str(snippets))
        return snippets



class Option:

    def __init__(self, **args):
        self.start = args.get('start') if args.get('start') else 0
        self.year_low = args.get('year_low')
        self.year_high = args.get('year_high')
